<?php
namespace OCA\Chores\Db;

use OCP\DB\QueryBuilder\IQueryBuilder;
use OCP\IDBConnection;
use OCP\AppFramework\Db\QBMapper;

class ChoreMapper extends QBMapper {

  public function __construct(IDBConnection $db) {
    parent::__construct($db, 'chores_chores', Chore::class);
  }

  public function findById(int $teamId, int $choreId): Chore {
    $qb = $this->db->getQueryBuilder();

    $qb->select('c.*')
       ->from($this->tableName, 'c')
       ->where(
         $qb->expr()->eq('team_id', $qb->createNamedParameter($teamId))
       )
       ->andWhere(
         $qb->expr()->eq('id', $qb->createNamedParameter($choreId))
       )
       ->andWhere(
         'deleted = false'
       )
       ;

    return $this->findEntity($qb);
  }

  public function findAll(string $teamId) {
    $qb = $this->db->getQueryBuilder();

    $qb->select('c.*')
       ->from($this->tableName, 'c')
       ->where(
         $qb->expr()->eq('team_id', $qb->createNamedParameter($teamId))
       )
       ->andWhere(
         'deleted = false'
       )
       ->orderBy('due', 'ASC')
       ;

    return $this->findEntities($qb);
  }

  public function markDeleted(int $choreId) {
    $qb = $this->db->getQueryBuilder();

    $qb->update('chores_chores')
       ->set('deleted', $qb->expr()->literal(true, IQueryBuilder::PARAM_BOOL))
       ->where(
         $qb->expr()->eq('id', $qb->createNamedParameter($choreId))
       )
       ;

    return $this->db->executeStatement($qb->getSQL(), $qb->getParameters());
  }

  public function updateDueDate(int $choreId, \DateTimeImmutable $dueDate) {
    $due = $dueDate->format('c');
    $qb = $this->db->getQueryBuilder();
    $qb->update($this->tableName)
       ->where(
         $qb->expr()->eq('id', $qb->createNamedParameter($choreId))
       )
       ->set('due', $qb->func()->greatest('due',
         $qb->createNamedParameter($due)
       ))
       ->execute();
  }

}
