<?php
namespace OCA\Chores\Db;

use OCP\IDBConnection;
use OCP\AppFramework\Db\QBMapper;

const ALPHABET = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

class InviteMapper extends QBMapper {

  public function __construct(IDBConnection $db) {
    parent::__construct($db, 'chores_invites', Invite::class);
  }

  public function findByTeamAndUser(int $teamId, string $userId): ?Invite {
    $qb = $this->db->getQueryBuilder();

    $qb->select('i.*')
       ->from($this->tableName, 'i')
       ->where(
         $qb->expr()->eq('i.team_id', $qb->createNamedParameter($teamId))
       )
       ->andWhere(
         $qb->expr()->eq('i.user_id', $qb->createNamedParameter($userId))
       )
       ;
    $found = $this->findEntities($qb);
    if (count($found) > 0) {
      return $found[0];
    } else {
      return null;
    }
  }

  public function createInvite(int $teamId, string $userId): Invite {
    $existing = $this->findByTeamAndUser($teamId, $userId);
    if ($existing !== null) {
      return $existing;
    }

    $randomId = implode("", array_map(function($i){
      return ALPHABET[random_int(0, 61)];
    }, range(0, 16)));
    $invite = Invite::fromParams([
      "teamId" => $teamId,
      "userId" => $userId,
      "id" => $randomId,
    ]);

    return $this->insert($invite);
  }

  public function findByTeam(int $teamId) {
    $qb = $this->db->getQueryBuilder();

    $qb->select('i.*')
       ->from($this->tableName, 'i')
       ->where(
         $qb->expr()->eq('i.team_id', $qb->createNamedParameter($teamId))
       )
       ;
    return $this->findEntities($qb);
  }

  public function findByUser(string $userId) {
    $qb = $this->db->getQueryBuilder();

    $qb->select('i.*', 't.team_name')
       ->from($this->tableName, 'i')
       ->where(
         $qb->expr()->eq('i.user_id', $qb->createNamedParameter($userId))
       )
       ->innerJoin('i', 'chores_teams', 't', 't.id = i.team_id')
       ;
    return $this->findEntities($qb);
  }

}
